use <Holder_PCB_LED.scad>;
use <Holder_generator_for_circular_optics.scad>;
use <Holder_LASER.scad>
use <detection_unit.scad>

$fn=100;
TH=300; //Through all Hole length

M3=3;
M3_screw=2.8; //to be threaded
M3_through=3.2; //traversant
M3_head=6; //IRL 5.5mm
M3_head_h=3.2; //IRL 3mm

M2=2;
M2_screw=1.8; //to be threaded
M2_through=2.2; //traversant

////////////////////////////////////////////////////////////////////////
//  Illumination_unit + 1st half of spectro_unit:
//  LED illumination, LASER illumination, light collection, pinhole, collimation, grating, Nema 17
////////////////////////////////////////////////////////////////////////

h_beam=22+10; //beam hight

d_c2c_LED=22*2; // distance center 2 center of the LED holder mounting holes

d_c2c_LASER=50; // distance center 2 center of the LASER holder mounting holes

d_c2c_cond=50; // distance center 2 center of the holder mounting holes
BF_cond=55; //back focale of condensor lens D42mm F40mm

d_c2c_pin=34; // distance center 2 center of the pinhole/slit holder mounting holes

d_c2c_coll=22; // distance center 2 center of the holder mounting holes
BF_coll=38; //back focale of collimation lens D12.7mm F40mm ()

d_LG=25; //distance of the collimation lens to the grating
d_GL=32; //distance of the grating to the focalisation lens

//d_c2c_foc=14; // distance center 2 center of the holder mounting holes
//BF_foc=28.7;  //back focale of foclisation lens D6mm F30mm (LA1700 Thorlabs)
d_c2c_foc=22;
BF_foc=38;

d_c2c_detection=25;
delta_hole_detection=8;

to_R_axis=[0,BF_cond+20+BF_cond+BF_coll+d_LG,0];

delta_angle=40; //delta angle between the two detection units (IR and Vis)
max_angle=60;   //Half-angle of maximum rotation
alpha=50;

//Nema17 Stepper Motor (42x42x38, M3 31x31mm )
///////////////////////////////////////////////////////////////////////////////////////////////////
SM_x=42; //Stepper-Motor side length
SM_h=38; //Stepper-Motor height
holes_stepper=[[1,1,0],[1,-1,0],[-1,1,0],[-1,-1,0]]*31/2;
module Nema17(){
    translate([ 0, 0, -SM_h/2])
    difference(){
        union(){
            cube([SM_x,SM_x,SM_h], center=true);
            translate([0,0,SM_h/2])   cylinder(h=2, d=22.5, center=false); //IRL=22mm
            translate([0,0,SM_h/2+2])   cylinder(h=21.5,d=5,  center=false);
        }
        for(v=holes_stepper) translate(v) cylinder(h=TH, d=M3_screw, center=true);
    }
}

//Base plate Spectro
///////////////////////////////////////////////////////////////////////////////////////////////////
e1=3.2;
module Base_plate_Spectro(e1) difference(){
//    //A4 paper size, for scale
//    translate([-210/2,0,-e1])
//    cube([210,297,e1],center=false);
    
//    //Reduced A4
//    translate([-180/2,0,-e1])
//    cube([180,280,e1],center=false);
    
    //Alu stock 200x300
    translate([-200/2,0,-e1])
    cube([200,280,e1],center=false);
    
    //Nema17 holes
    translate(to_R_axis) cylinder(h=TH, d=22, center=true);
    for(v=holes_stepper) translate(v+to_R_axis) cylinder(h=TH, d=M3_through, center=true);
   
    //End-Switch-holes
    holes_endsw=[[40,-31/2,0],[60,-31/2,0],[-40,-31/2,0],[-60,-31/2,0]];
    for(v=holes_endsw) translate(v+to_R_axis) cylinder(h=TH, d=M3_through, center=true);
   
        
    //Main PCB holes: 50x152
    translate([32+4,1+4,0]){
        translate([0,   0,  0]) cylinder(h=TH,d=M3_screw,center=true);
        translate([50,  0,  0]) cylinder(h=TH,d=M3_screw,center=true);
        translate([0,   152,0]) cylinder(h=TH,d=M3_screw,center=true);
        translate([50,  152,0]) cylinder(h=TH,d=M3_screw,center=true);
    }
        
    //Holes for optics Holder :
    for(m=[-1,+1]){
        //LED_holder
        translate([m*d_c2c_LED/2,5.57,0]) cylinder(h=TH,d=M2_screw,center=true);
        //LASER
        translate([m*d_c2c_LASER/2,20,0]) cylinder(h=TH,d=M2_screw,center=true);
        //condenser
        translate([m*d_c2c_cond/2,BF_cond   ,0]) cylinder(h=TH,d=M2_screw,center=true);
        translate([m*d_c2c_cond/2,BF_cond+20,0]) cylinder(h=TH,d=M2_screw,center=true);
        //pinhole
        translate([m*d_c2c_pin/2,BF_cond+20+BF_cond,0]) cylinder(h=TH,d=M2_screw,center=true);
        //collimation
        translate([m*d_c2c_coll/2,BF_cond+20+BF_cond+BF_coll,0]) cylinder(h=TH,d=M2_screw,center=true);
    }
    
    //Master holes (for opto-plate sub-assembly)
    echo("opto-plate sub-assembly holes from sample edge:");
        //between laser and first condenser
        Y1=(20+BF_cond)/2;
        echo(Y1);
        //between laser and pinhole
        Y2=(BF_cond+20 + BF_cond+20+BF_cond)/2;
        echo(Y2);
        //between pinhole and collimation
        Y3=(BF_cond+20+BF_cond + BF_cond+20+BF_cond+BF_coll)/2;
        echo(Y3);
    for(m=[-1,0,+1]){
        translate([m*d_c2c_cond/2,Y1,0]) cylinder(h=TH,d=M3_screw,center=true);
        translate([m*d_c2c_cond/2,Y2,0]) cylinder(h=TH,d=M3_screw,center=true);
        translate([m*d_c2c_cond/2,Y3,0]) cylinder(h=TH,d=M3_screw,center=true);
    }
}

//Base plate Rotation
///////////////////////////////////////////////////////////////////////////////////////////////////
e2=7;
module Base_plate_Rotation(alpha) difference(){
    hull() {
        rotate(alpha)                translate([-16/2,-5/2-4,0])  cube([16,3+6,e2],center=false);
        rotate(alpha-delta_angle/2)  translate([0,d_GL+BF_foc,0]) cylinder(h=e2,d=40,center=false);
        rotate(alpha+delta_angle/2)  translate([0,d_GL+BF_foc,0]) cylinder(h=e2,d=40,center=false);
    }
    
    //stepper motor axis
    intersection(){
        cylinder(h=TH, d=5+0.15, center=true);
        translate([0,0.5,0]) cube([5,5,TH],center=true);
    }
    //set_screw
    translate([0,-TH/2,e2/2]) rotate([90,0,0]) cylinder(d=2.7,h=TH,center=true);
    
    //ball transfert holes
    d_ball=12.8+0.2;
    translate([0,0,0]){
        rotate(alpha)                translate([0,d_GL+12,0])      cylinder(h=TH,d=d_ball,center=true);
        rotate(alpha-delta_angle/2)  translate([0,d_GL+BF_foc,0]) cylinder(h=TH,d=d_ball,center=true);
        rotate(alpha+delta_angle/2)  translate([0,d_GL+BF_foc,0]) cylinder(h=TH,d=d_ball,center=true);
    }
    //focalisation lens holes
    //for(i=[-1,+1])
    for(i=[1]) rotate(alpha+i*delta_angle/2)
        for(m=[-1,+1]) translate([m*d_c2c_foc/2,d_GL,0])
            cylinder(h=TH,d=M2_screw,center=true);       
    //detection units holes
    for(i=[-1,+1]) rotate(alpha+i*delta_angle/2)
        for(m=[-1,+1]) translate([m*d_c2c_detection/2,d_GL+BF_foc+1.6+delta_hole_detection,0])
            cylinder(h=TH,d=M2_screw,center=true);   
}
*Base_plate_Rotation(0);

//roller / Ball transfert
///////////////////////////////////////////////////////////////////////////////////////////////////
//h_roller=6;
module ball_transfert1() { //ALWAYSE ref 11MI-05-17
    cylinder(h=2.5,d=2,center=false);
    translate([0,0,-h_roller]) cylinder(h=h_roller,d=8,center=false);
}
h_roller=10.4-7; //3.4mm
module ball_transfert2() { //NIKEYE Store (AliExpres) ref "NL-8H"
    cylinder(h=7,d=13,center=false);
    translate([0,0,-h_roller]) cylinder(h=h_roller,d=15,center=false);
}

//Baffles
///////////////////////////////////////////////////////////////////////////////////////////////////
module baffle1(){
    L=BF_cond-10.07;
    difference(){
        union(){
            //outside cone
            e=3;
            cylinder(h=L,d1=5+2*e,d2=40+2*e,center=false);
            //holder on holes between laser and condenser
            L_foot=BF_cond-(BF_cond+20)/2;
            difference(){
                translate([-(50+2*4)/2,0,-(L_foot+4)+L]) cube([50+2*4,25,L_foot+4],center=false);
                for(m=[-1,+1]) translate([m*50/2,0,L-L_foot])
                    rotate([90,0,0]) cylinder(h=TH,d=M3_through,center=true);
            }
        }
        //main light beam
        cylinder(h=L,d1=5,d2=40,center=false);
        //trim central screw
        translate([0,TH/2+25-3,0]) cube([M3_head,TH,TH],center=true);
        //trim from condenser edge
        delta_trim=4;
        translate([0,0,TH/2+L-delta_trim]) cube([TH,TH,TH],center=true);
    }
}
*baffle1();



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  OZI assembly  ///////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

module ozi(){
    //Holder for sample
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    translate([0,1.2/2,h_beam]) rotate([90,0,0]) translate([0,0,0])
    color("black",0.5)
    render()
    Holder_sample();


    //LEDs
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    translate([0,6.76+1.2/2,h_beam]) rotate([90,0,0]){
        //PCB and LED
        %PCB_LED_flat();
        LED_light();
        %Sample_diaf();
        //Holder for PCB LED
        color("orange",0.8) Holder_PCB_LED();   //! h_beam as inside parameter of Holder_PCB_LED.scad
    }

    //Baffle
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    translate([0,10.07,25+7]) rotate([-90,0,0]) baffle1();

    //LASER
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    color("blue",0.8)
    translate([0,20,h_beam]) rotate([90,0,180])
    Holder_LASER();
    
    


    //Condensers
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    
    color("orange",0.8)
    translate([0,BF_cond,h_beam]) rotate([90,0,180])
    lens_holder(    D_lens=42+0.0,
                    e_edge_lens=2.03,  
                    e_max_lens=8.2,
                    h_beam=h_beam);

    color("orange",0.8)
    translate([0,BF_cond+20,h_beam]) rotate([90,0,0])
    lens_holder(    D_lens=42+0.0,
                    e_edge_lens=2.03,  
                    e_max_lens=8.2,
                    h_beam=h_beam);


    //pinhole
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    color("blue",0.8)
    translate([0,BF_cond+20+BF_cond,h_beam]) rotate([90,0,0])
    lens_holder(    D_lens=25.4,
                    e_edge_lens=3,  
                    e_max_lens=3,
                    h_beam=h_beam);
                    
    //Collimation
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    color("orange",0.8) translate([0,BF_cond+20+BF_cond+BF_coll,h_beam]) rotate([90,0,180])
    lens_holder(      D_lens=12.7+0.0, ////LA1304
                      e_edge_lens=1.8,  
                      e_max_lens=2.8,
                      h_beam=h_beam);
                    
    //scanning axis
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    //translate(to_R axis) cylinder(h=100,d=5,center=true);
    
    
    //Grating
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    module Grating(alpha){ //grating 15€
        h_grating=50;
        translate([0,0,h_grating/2]) rotate([0,90,90+alpha]) difference(){
            //frame
            cube([h_grating,50,2], center=true);
            //clear apperture
            cube([23,35,TH], center=true);
        }
    }

    //for(alpha=[-20,0,20]) translate(to_R_axis+[0,0,10]) Grating(alpha);  
    for(alpha=[0]) translate(to_R_axis+[0,0,10]) Grating(alpha); 
        
    //holder grating
    
    

    //Detection/scanning units !! with a smaller h_beam=22 !!
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    
    module Holder_lens_foc(){
        translate([0,0,h_beam]) rotate([90,0,0])
//        lens_holder(  D_lens=6+0.0,  //LA1700
//                      e_edge_lens=1.5,  
//                      e_max_lens=1.8,
//                      h_beam=22);
        lens_holder(  D_lens=12.7+0.0, ////LA1304
                      e_edge_lens=1.8,  
                      e_max_lens=2.8,
                      h_beam=22);
    }
        
    module scanning_unit_elements(beta){
        translate(to_R_axis) rotate(beta){
            //lens+IR
            rotate(0){
                translate([0,d_GL,0]) Holder_lens_foc();
                translate([0,d_GL+BF_foc+1.6,h_beam]) detection_unit_assembly(); //h_beam=22 in da script
            }
            //lens+Vis
            rotate(delta_angle){
                translate([0,d_GL,0]) Holder_lens_foc();
                translate([0,d_GL+BF_foc+1.6,h_beam]) detection_unit_assembly();
            }
        }
    }
    
    
    scanning_unit_elements(beta=alpha-delta_angle/2);

    //max rotation -
    color("grey",0.2) render() scanning_unit_elements(beta=-max_angle-delta_angle);


    //Ball transfert
//    ///////////////////////////////////////////////////////////////////////////////////////////////////
//    color("black",0.5) translate(to_R_axis){
//        rotate(alpha-delta_angle/2)  translate([0,d_GL+BF_foc,h_roller]) ball_transfert1();
//        rotate(alpha+delta_angle/2)  translate([0,d_GL+BF_foc,h_roller]) ball_transfert1();
//    }
    color("black",0.5) translate(to_R_axis){
        rotate(alpha-delta_angle/2)  translate([0,d_GL+BF_foc-5,h_roller]) ball_transfert2();
        rotate(alpha+delta_angle/2)  translate([0,d_GL+BF_foc-5,h_roller]) ball_transfert2();
    }


    //Base plate Rotation
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    color("aqua",0.4) translate(to_R_axis) translate([0,0,h_roller]) Base_plate_Rotation(alpha);
    //max rotation -
    color("grey",0.2) translate(to_R_axis) translate([0,0,h_roller]) Base_plate_Rotation(-max_angle-delta_angle/2);
    
    
    //Main PCB
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    module main_pcb() difference(){
        X=100;
        Y=160;
        cube([X,Y,1.6]);
        //mounting holes
        holes=[ [4, 4,  0],
                [4, Y-4,0],
                [54,4,  0],
                [54,Y-4,0]];
        for(v=holes) translate(v) cylinder(h=TH,d=M3,center=true);
        //FFC connection (to PCB hub)
        translate([10,Y-15,0]) cube([5,10,TH],center=true);
        translate([20,Y-15,0]) cube([5,10,TH],center=true);
        //FFC connection (to PCB LED)
        translate([10,20,0]) rotate(90) cube([5,10,TH],center=true);
    }
    color("green",0.5) translate([32,1,0.1]) render() main_pcb();
    
    //Nema17
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    color("purple",alpha=0.8) translate(to_R_axis+[0,0,-e1]) Nema17();
    //Base plate Spectro
    ///////////////////////////////////////////////////////////////////////////////////////////////////    
    color("white",alpha=0.4) Base_plate_Spectro(e1=e1);
    
}




////////////////////////////////////////////////////////////////////////
//  To be seen from the "RIGHT VIEW" ctrl+7
////////////////////////////////////////////////////////////////////////

//side view (main)
!ozi();

//Right view
//translate([0,-210/2-5,0]) rotate([0,0,90]) ozi();

//top view
//translate([0,0,-210/2-8-38-5]) rotate([0,90,0]) ozi();


////////////////////////////////////////////////////////////////////////
//  mini plate holder
////////////////////////////////////////////////////////////////////////

//illumination
color("black",alpha=0.5) translate([0,0,10]) difference(){
    Base_plate_Spectro(e1=7.4);
    //trim long sides
    for(m=[-1,+1,]) translate([m*(TH+50+2*5)/2,TH/2-1,0]) cube([TH,TH,TH],center=true);
    //trim short side
    translate([0,TH/2+110,0]) cube([TH,TH,TH],center=true);
    //trim center
    translate([0,TH/2+110-25,0]) cube([40,TH,TH],center=true); 
}

//pin hole collimation
color("black",alpha=0.5) translate([0,0,7.4+0.1]) difference(){
    Base_plate_Spectro(e1=7.4);
    //trim long sides
    for(m=[-1,+1,]) translate([m*(TH+50+2*5)/2,TH/2-1,0]) cube([TH,TH,TH],center=true);
    //trim short sides
    translate([0,-TH/2+110-20,0]) cube([TH,TH,TH],center=true);
    translate([0,TH/2+173,0]) cube([TH,TH,TH],center=true);
    //trim around center
    //translate([0,TH/2+110-25,0]) cube([40,TH,TH],center=true); 
    for(m=[-1,+1,]) translate([m*(TH+30)/2,-TH/2+110+10,0]) cube([TH,TH,TH],center=true);
    for(m=[-1,+1,]) translate([m*(TH+35)/2,TH/2+173-10,0]) cube([TH,TH,TH],center=true);
}




////////////////////////////////////////////////////////////////////////
//  To be exported
////////////////////////////////////////////////////////////////////////

//2D
projection()
translate([0,0,0])
{
    translate([200/2+50,0,0]) Base_plate_Spectro(e1);
    //translate([0,0,-e2]) Base_plate_Rotation(0);
}

//3D
translate([0,0,0])
{
    for(i=[1,2,3]) translate([-20*i,0,6]) ball_transfert1();
    translate([0,0,0]) Base_plate_Rotation(0);
}

//2D PCB main
*projection()
translate([0,0,0])
{
    main_pcb();
}