// PCB LED v2 Holder by Mej for the OZIRMA Recycling initiative

FN=100; // default facets for cylinders
TH=200; //Through all Hole

M3=3;
M3_screw=2.7; //to be threaded
M3_through=3.2; //traversant
M3_head=6.3; //head screw (IRL 5.5)

M2=2;
M2_screw=1.8; //to be threaded
M2_through=2.2; //traversant
M2_head=4.5; //head screw (IRL 3.8)

//////////
// Design parametrers of LED_PCB (on FLAT pcb, as opposed to FOLDED pcb)
//////////

d1=3.54; //distance from center (<=> diaph center) to ligne of folding
d2=d1+2; //distance from center to LED
d3=d2+3; //distance from center to M2 attachment hole
d4=d3+2; //distance from center to end of PCB

D_diaph=4; //central diaphragme Diameter
e=1.5; //Rainure width
E=2; //Rainure length

e_PCB=1; //PCB width

directions=[180,60,-60]; //position [°] of satelites, referenced from y axis
//45 = folding angle [°]


//////////
// Design parametrers of hoders
//////////

e_holder=2.6; //epaisseur of holder under PCB
e_sample=1.2; //epaisseur du sample holder
D_sample=3; //diameter of sample holder diafragme

h_holder=e_holder+e_PCB+6.36-e_sample;
y_edge=d1+(d4-d1)*cos(45); //=22-8.49..~13.5
w_holder=0.5;//extra width of PCB holder on side of each satelite
//h_beam=22+10;
h_beam=25;
W_holder=[h_beam-y_edge,12,12]; //extra width of PCB holder on the outside back, for each direction. ! 22=h_beam !


M2_screw_length=4; //Adjust here the length of the screww that attach the PCB on the holder.
                   // ! Max is 7-8 mm. ! 



// PCB LED holder
////////////////////////////////////////////////////////////////////////////////////////////////////
module Holder_PCB_LED(h_beam){
    difference(){
        translate([0,0,TH/2/2-e_holder])   cube([TH,TH,TH/2],center=true);
                                        //cylinder(h=10,r=y_edge/cos(30),center=true, $fn=6);
        
        //Main cut
        hull() translate([0,0,e_PCB/2]){
            //Hexagon to ligne of folding, on PCB back base
            cylinder(h=e_PCB,r=d1/cos(30),center=true, $fn=6);
            //Hexagon to end of PCB, AFTER folding, on PCB back top edge
            d=d4-d1;
            r=d1+d*cos(45);
            translate([0,0,d*sin(45)])      cylinder(h=e_PCB,r=r/cos(30),center=true, $fn=6);
            translate([0,0,TH])             cylinder(h=e_PCB,r=r/cos(30),center=true, $fn=6);
        }
        //Play on main cut base (ligne of folding)
        play=2*0.5; //clearance on PCB_edges. Adjust for folding radius
        translate([0,0,e_PCB/2]) cylinder(h=e_PCB,r=(d1+play)/cos(30),center=true, $fn=6);
        
        //M2 attachment holes, AFTER folding
        d=d3-d1;
        delta_folding=+0.5; // ! durty!
        for(alpha=directions) rotate(alpha) translate([0,d1+d*cos(45)+delta_folding,d*sin(45)]) rotate([45,0,0])
        {
            cylinder(h=TH,d=M2_through,center=true, $fn=FN);
            //head screw
            rotate([180,0,0]) translate([0,0,M2_screw_length-e_PCB-0.6]) cylinder(h=TH,d=M2_head, center=false, $fn=FN);
        }
        
        //trim to satelite PCB...
        for(i=[0,1,2]) rotate([0,0,directions[i]]){
            //... side edges
            for(m=[-1,1]) translate([m*(TH/2+5/2+w_holder),TH/2+d1,0])   cube([TH,TH,TH],center=true);
            //... back edge
            d=d4-d1;
            delta_up=-0.146;        // ! durty!
            translate([0,d1+d*cos(45),d*sin(45)+e_PCB/2]) rotate([0,45,90])
                translate([delta_up,0,0]) cube([1,10,3],center=true);
            //... back edge + W_holder
            translate([0,TH/2+y_edge+W_holder[i],0])   cube([TH,TH,TH],center=true);
        }
        //trim to central PCB sides
        for(alpha=directions) rotate(alpha+180) translate([0,TH/2+4+0.3,0])
            cube([6,TH,TH],center=true);
        //trim top
        translate([0,0,TH/2-e_holder+h_holder])            cube([TH,TH,TH],center=true);
        
        //Central diafragme
        cylinder(h=TH,d=D_diaph+1,center=true, $fn=FN);
        
        //mounting holes for sample holder
        for(alpha=directions) rotate(alpha) translate([0,y_edge+W_holder[1]-3,0])
            cylinder(h=TH,d=M2_screw,center=true, $fn=FN);
    }
    
    // Handle for OZIRMA
     difference(){
        y_h=y_edge+W_holder[0];
        union(){
            translate([+(5/2+w_holder),-y_h,-e_holder])                    cube([22,W_holder[1],h_holder],center=false);
            translate([-(5/2+w_holder),-y_h,-e_holder]) mirror([1,0,0])    cube([22,W_holder[1],h_holder],center=false);
        }
        //mounting holes on spectrometer
        for(m=[-1,1]) translate([m*22,0,-e_holder+h_holder/2]) rotate([90,0,0])
            cylinder(h=TH,d=M2_through,center=true, $fn=FN); 
    }
    
//    //re-enforcement (backbone)
//    x=50;
//    #translate([0,-5,3/2-e_holder]) difference(){
////         cube([x,x,3],center=true);
////         cube([x-5,x-5,11],center=true);
//           cylinder(h=3,d=x,center=true,$fn=100);
//           cylinder(h=3+1,d=x-5,center=true,$fn=100);
//    }
}


// Sample holder
////////////////////////////////////////////////////////////////////////////////////////////////////
module Holder_sample(){
    difference(){
        //rotate(30) cylinder(h=e_sample,r=21,center=true, $fn=6);
        rotate(30) cylinder(h=5,r=21,center=true, $fn=6);
        //Diaphragm hole
        //cylinder(h=TH,d=D_sample,center=true, $fn=FN);
        cylinder(h=TH,d=20.05,center=true, $fn=FN);
        //mounting holes
        for(alpha=directions) rotate(alpha) translate([0,y_edge+W_holder[1]-3,0])
            cylinder(h=TH,d=M2_through,center=true, $fn=FN); 
        //watermark
        //linear_extrude(height=e_sample) translate([0,-(y_edge+W_holder[1]-3)/2])
            //text("OZIRMA", halign="center", valign="center", font="Cambria:style=Bold ",size=5);
    }
}

module Holder_sample_Faience(){
    h=7+1;
    //main
    *difference(){
        rotate(30) cylinder(h=h,r=21,center=true, $fn=6);
        //PCB cut
        translate([8,-TH,-TH/2]) cube([TH,TH,TH],center=false);
        //Faience poket
        translate([0,0,(TH-h)/2+1])  cube([18,18,TH],center=true);
        //Diaphragm hole
        cylinder(h=TH,d=D_sample,center=true, $fn=FN);
        //mounting holes (to LED holder)
        for(alpha=directions) rotate(alpha) translate([0,y_edge+W_holder[1]-3,0])
            cylinder(h=TH,d=M2_through,center=true, $fn=FN); 
        //mounting hole (to faience back-push)
        for(alpha=directions) rotate(alpha-20) translate([0,15,0])
            cylinder(h=TH,d=M2_screw,center=true, $fn=FN); 
    }
    //back-push
    translate([40,0,1-h/2]) difference(){
        rotate(30) cylinder(h=2,r=21,center=true, $fn=6);
        //PCB cut
        translate([8,-TH,-TH/2]) cube([TH,TH,TH],center=false);
        //clear holes (to LED holder)
        for(alpha=directions) rotate(alpha) translate([0,y_edge+W_holder[1]-3,0])
            cylinder(h=TH,d=4,center=true, $fn=FN); 
        //mounting hole (to faience back-push)
        for(alpha=directions) rotate(alpha-20) translate([0,15,0])
            cylinder(h=TH,d=M2_through,center=true, $fn=FN); 
    }
}
!Holder_sample_Faience();

// Core PCB, flat (BEFORE folding)
////////////////////////////////////////////////////////////////////////////////////////////////////
module PCB_LED_flat(){
    translate([0,0,e_PCB/2]){
    //core PCB design
    difference(){ 
        union(){
            //Central "hexagone"
            cylinder(h=e_PCB,r=4/cos(30),center=true, $fn=6);//R to r conversion: wikipedia.org/wiki/Hexagon
            //Satelites PCBs
            a=d3-d1+2; //=(3+2)+2=7 coté
            for(alpha=directions) rotate(alpha) translate([0,a/2+d1,0]) difference(){       
                cube([5,a,e_PCB],center=true);       
                //Attachment hole
                translate([0,-a/2+d3-d1,0]) cylinder(h=TH,d=M2_screw,center=true, $fn=FN);           
                //LED position
                translate([0,-a/2+d2-d1,0]) cylinder(h=TH,r=0.2,center=true, $fn=FN);
            }
        }
        //Diafragme
        cylinder(h=TH,d=D_diaph,center=true, $fn=FN);
        //Rainures
        long=5;
        for(alpha=directions, m=[+1,-1]) rotate(alpha) translate([m*(long+E)/2,d1,0])
            cube([long,e,e_PCB*4],center=true);
    }
    //Connector area
    x1=21-7.88; // Connector area, big square
    x2=7.88-5; // small square
    translate([x1/2+x2+5/2,10/2-d1-e/2-6.25]){
        difference() {
            cube([x1,10,e_PCB],center=true);
            translate([-x1/2,10/2,0]) rotate(45) cube([5.402,5.303,TH],center=true);
        }
        translate([-x1/2-x2,-10/2,-e_PCB/2]) cube([x2,6.25,e_PCB]);
    }
    }
}

//LED light, AFTER folding
////////////////////////////////////////////////////////////////////////////////////////////////////
module LED_light(){
    d=d2-d1;
    Theta=30; //half angle emission
    colors=["red","green","blue"]; 
    for(i=[0,1,2]) color(colors[i],0.5){
         render() difference(){
            rotate(directions[i]) translate([0,d1+d*cos(45),e_PCB+d*sin(45)]) rotate([45,0,0])
                translate([0,0,e_PCB/2]) cylinder(h=10,d1=0.3, d2=2*10*tan(Theta), center=false, $fn=FN);
            //trim top
            translate([0,0,TH/2-e_holder+h_holder+e_sample-0.01])    cube([TH,TH,TH],center=true);
        }
    }
}

//IRL sample diaf visualisation
////////////////////////////////////////////////////////////////////////////////////////////////////
module Sample_diaf(){
    translate([0,0,-e_sample/2+e_PCB+6.36]) difference(){
        cylinder(h=e_sample+0.1,d=D_sample+5,center=true, $fn=FN);
        cylinder(h=e_sample+1,d=D_sample,center=true, $fn=FN);
    }
}



////////////////////////////////////////////////////////////////////////////////////////////////////
// Assembly
////////////////////////////////////////////////////////////////////////////////////////////////////


//PCB and LED
%PCB_LED_flat();
%LED_light();
%Sample_diaf();


//Holder for PCB LED
*Holder_PCB_LED();


//Holder for sample
//translate([-40,0,e_sample/2-e_holder]) //placement for 3D printing
//translate([0,0,-e_sample/2+e_PCB+6.36]) //placement IRL
translate([0,0,5/2])
Holder_sample();

//holder sample adapter
translate([0,35,0])
difference(){
    cylinder(d=20-0.2,h=20,center=false, $fn=FN);
    translate([0,0,1])
        cylinder(d=20-0.2-2,h=20,center=false,$fn=FN);
    //Diaphragm hole
    cylinder(h=TH,d=D_sample,center=true, $fn=FN);
    //screw holes
    for(alpha=directions) rotate(alpha) translate([0,10-4,0])
            cylinder(h=TH,d=M2_screw,center=true, $fn=FN); 
}




