//All distance in [mm]. All angles in [°]


//General parameters
/////////////////////
TH=100;
$fn=100;

M2=2;
M2_screw=1.8; //to be threaded
M2_through=2.2; //traversant
M2_head=5; //head of screw or washer outside diameter



//////////////////////////////////////////////////////////////////////

module lens_holder(D_lens,e_edge_lens,e_max_lens,h_beam){
    //D_lens        Diameter of the optic 
    //e_edge_lens   thickness at the edge of the optic
    //e_max_lens    max thickness of the optics (for a plano convexe lens, it is the center thickness)
    //h_beam        height of beam
    
    // Origine at the center of the optic.
    // Z axis colinear with the optic axis
    // (xy) plane = "bottom" of the optic = mounting axis of on holder' foot
    
    echo("\n\nGenerating holder for optic of diameter",D_lens);

    e_clearance_holder=1.3; //minimal thickness of the holder
    l_thread_top=3; //distance to be threaded for the lens_holding screw
    l_clear_top=1; //top space for optic to slide into place
    h_foot=5; //height of the foot
    
    /////////////////////////////
    
    d_c2c_0=D_lens+2*e_clearance_holder+M2_head; //distance center-to-center of the mounting
    echo("d_c2c_0=",d_c2c_0);
    d_c2c=d_c2c_0-d_c2c_0%2+2;
    //d_c2c=d_c2c_0;
    echo("d_c2c=",d_c2c);
    
    h=e_clearance_holder+M2_through/2+e_max_lens+e_clearance_holder; //thickness (height along Z axis) of holder
    echo("z height=",h,"\n");

    difference(){
        hull(){
            //head
            d=D_lens+2*e_clearance_holder;
            if(d/2>h_beam) echo("!!!!!!!!!!!!!!!!!!!!!!! outisde radius of holder=",d/2,"> h_beam=",h_beam);
            translate([0,0,-e_clearance_holder-M2_through/2]) hull(){
                cylinder(h=h,d=d,center=false);
                translate([0,l_clear_top,0]) cylinder(h=h,d=d,center=false);
            }
            //hat
            translate([0,l_thread_top/2+D_lens/2+l_clear_top,h/2-e_clearance_holder-M2_through/2])
                cube([M2+2*e_clearance_holder,l_thread_top,h],center=true);
            //feet            
            R_clear=(M2_through+2*e_clearance_holder)/2;
            for(i=[-1,+1]) translate([i*(d_c2c/2+R_clear),-h_beam+h_foot/2,-R_clear])
                cylinder(h=h, d=h_foot, center=false);
            
            //translate([d_c2c/2,0,0]) rotate([-90,0,0]) cylinder(h=TH, d=M2_head, center=true);
        }
        //clear apperture
        cylinder(h=TH,d=D_lens-e_clearance_holder,center=true);
        //cylinder(h=TH,d=D_lens-2*e_clearance_holder,center=true);
        //optic hole
        hull(){
            //optic only
            cylinder(h=TH,d=D_lens,center=false);
            //space on top of optic
            translate([0,l_clear_top,0]) cylinder(h=TH,d=D_lens,center=false);
        }
        //lens holding screw
        translate([0,0,e_edge_lens/2]) rotate([-90,0,0]) cylinder(h=TH,d=M2_screw,center=false);
        //mounting holes
        for(i=[0,1]) rotate([0,i*180,0]) translate([d_c2c/2,0,0]) rotate([-90,0,0]){
            //screw
            cylinder(h=TH,d=M2_through,center=true);
            //head
            translate([0,0,-h_beam+h_foot]) cylinder(h=TH,d=M2_head,center=false);
        }      
        //trim side
        for(i=[0,1]) rotate([0,i*180,0]) translate([TH/2+D_lens/2+e_clearance_holder,TH/2-h_beam+h_foot,0])
            cube([TH,TH,TH],center=true); 
        
        //trim bottom (only necessary if hèbeam too low
        translate([0,-TH/2-h_beam,0]) cube([TH,TH,TH],center=true); 
    }
}


//Holders for...
/////////////////////

h_beam=25;

//Condenser
for(i=[0,1]) translate([i*64,0,0])
lens_holder(    D_lens=42+0.0,
                e_edge_lens=2.03,  
                e_max_lens=8.2,
                h_beam=h_beam);

//Pinhole Thorlabs
translate([-10,-45,0])
lens_holder(    D_lens=25.4,
                e_edge_lens=3,  
                e_max_lens=3,
                h_beam=h_beam);
//custom hole holder
!translate([-40,-45,0]) difference(){
    cylinder(d=25.4,h=3,center=true);
    //central hole
    cylinder(d=3,h=TH,center=true);
    //poket for metal shit
    cylinder(d=15,h=TH,center=false);
    //mounting holes
    for(alpha=[0,120,240]) rotate(alpha) translate([0,5,0]) cylinder(d=1.8,h=TH,center=true);
}

////collimation with LA1304
translate([32,0,0]) rotate(180)
lens_holder(    D_lens=12.7+0.0,
                e_edge_lens=1.8,  
                e_max_lens=2.8,
                h_beam=h_beam);


h_beam_foc=22;
//focalisation with LA1700
for(i=[0:3]) translate([26+i*25,-45,0])
lens_holder(    D_lens=6+0.0,
                e_edge_lens=1.5,  
                e_max_lens=1.8,
                h_beam=h_beam_foc);

//focalisation with LA1304
*for(i=[0]) translate([26+i*25,-45,0])
lens_holder(    D_lens=12.7+0.0,
                e_edge_lens=1.8,  
                e_max_lens=2.8,
                h_beam=h_beam_foc);



