$fn=100; //Set to 100 for pretty openscad rendering.
TH=50; //Through all Hole

//////////
// Grating holder
//
// Grating plane = (y,z) x=0   <=> axis of rotation
// Screw onto Nema17 monting holes (M3, at the corner of a 31mm sided square)
M3_through=3.1;
d_shaft=5.2;
Nema17_c2c=31;
//////////


//////////
// Design parametrers 
//////////
h_grating=11+2; // hight of the bottom of the grating
e=2; //minimal wall thickness
e_z=1; //minimal layer thickness

//Gratings parameters: [ref,    thickness, delta from center, 'aka']
gratings_param=[       [1,      3.75-0.2,      (3.75-0.2)/2, "holo"],
                       [2,      2,         0,                "line"],
                       [3,      0.4,       0.2,              "PET"]];

alpha=-10;


//filter
h_beam=34; 
d_filter=25.4;
e_filter=3.5;

//////////
// Grating holder
//////////

module holder_grating(p){
    echo("grating ref=",p[0],p[3]);
    echo("alpha=",alpha);
    
    //Gratings Holder outside shape:
    X=Nema17_c2c/2  +max(d_shaft/2,p[1]/2+p[2])+e  +M3_through/2+e +3; //Grating holder width
    Y=40; //length
    Z=h_grating+8; //height

    difference(){
        //body
        //translate([X/2-max(d_shaft/2,p[1]/2+p[2])-e,0,Z/2-h_grating]) cube([X,Y,Z],center=true);
        translate([-X/2+Nema17_c2c/2+M3_through/2+e,0,Z/2-h_grating]) cube([X,Y,Z],center=true);
        
        //motor axis
        translate([0,0,-TH/2-e_z/2]) cylinder(d=d_shaft,h=TH,center=true);
        
        //Monting holes
        translate([Nema17_c2c/2,+Nema17_c2c/2,0])   cylinder(d=M3_through,h=TH,center=true);
        translate([Nema17_c2c/2,-Nema17_c2c/2,0])   cylinder(d=M3_through,h=TH,center=true);
        
        //bottom trim
        translate([-TH/2+Nema17_c2c/2-M3_through/2-e,0,-TH/2-e_z]) cube([TH,TH,TH],center=true);
         
        //grating slit
        rotate(-alpha)
        translate([-p[2],0,TH/2]) cube([p[1],TH,TH],center=true);
    }

}

//////////
// Filter holder
//////////

module holder_filter_body(){ //whitout filter slit nor clear apperture
    X=M3_through+2*e;
    Y1=40;
    Y2=Nema17_c2c-M3_through-2*e; 
    difference(){
        //body
        union(){
            translate([-X/2,-Y1/2,0])    cube([X,Y1,2],center=false);
            translate([-X/2,-Y2/2,0])    cube([X,Y2,10],center=false);
        }
        //Monting holes
        translate([0,+Nema17_c2c/2,0])   cylinder(d=M3_through,h=TH,center=true);
        translate([0,-Nema17_c2c/2,0])   cylinder(d=M3_through,h=TH,center=true);
    }
}

module holder_filter(){ //in assembly position
    difference(){
        translate([Nema17_c2c/2,0,8])   holder_filter_body();
        //filter slit
        translate([Nema17_c2c/2,0,-h_grating+h_beam]) rotate([0,90,0]) cylinder(d=d_filter,h=e_filter,center=true);
        //clear apperture
        translate([0,0,-h_grating+h_beam]) rotate([0,90,0]) cylinder(d=20,h=TH,center=true);
    }
}


//////////
// Assembly
//////////

holder_grating(gratings_param[0]);

holder_filter();

//filter
#translate([Nema17_c2c/2,0,-h_grating+h_beam])
    rotate([0,90,0]) cylinder(d=d_filter,h=e_filter,center=true);


//////////
// 3D print
//////////
module 3D_print(){
    translate([-Nema17_c2c/2,0,0]) rotate([180,0,0]) holder_grating(gratings_param[0]);
    translate([0,0,-8*2]) holder_filter();
}

!3D_print();
 

