TH=100;
$fn=100;

M2=2;
M2_screw=1.8; //to be threaded
M2_through=2.2; //traversant

M3_screw=2.8; //to be threaded

//////////////////////////////////////////////////////////////////////////////////
// Scanning unit of OZI
//////////////////////////////////////////////////////////////////////////////////

holes = [[-23/2+2.5, 0, -6.5+2.5],
         [+23/2-2.5, 0, -6.5+2.5],
         [0,         0, -6.5+2.5+18]]; //from PhD center axis, in (x,z) plane
     
     
c_to_c=16; // center to center distance of sliding axis (shafts and bushing)

D_bushing=5; //external diameter of bushing ring
e_bushing=2.5; //thikness of bushing ring
wall_bushing=2; //thikness around bushing ring

D_shaft=2; //external diameter of shaft

thickness=2.8; //3D plates thickness
e1=thickness+D_bushing+2*wall_bushing; //total thickness of PhD-PCB holder
delta_holders=1; //space between the 2 holders (has to accomodate for PhD PCB holding screws)

d_spring=4+2*1; //spring diameter + clearance

h_beam=22;

// PhD PCB
//////////////////////////////////////////////////////////////////////////////////
module PCB_PhD(){
    difference([]){
        cube([23,23,1.6]);
        //mounting holes
        translate([2.5,      2.5,    0])      cylinder(h=TH,d=M2,center=true);
        translate([2.5+18,   2.5,    0])      cylinder(h=TH,d=M2,center=true);
        translate([2.5+18/2, 2.5+18, 0])      cylinder(h=TH,d=M2,center=true);
        //PhD axis
        translate([2.5+18/2, 6.5,    0])      cylinder(h=TH,d=1,center=true);
    }
     %translate([2.5+18/2, 6.5,    10/2])      cylinder(h=10,d=0.3,center=true);
}

// Shafts
//////////////////////////////////////////////////////////////////////////////////
module Shafts(){
    for(m=[-1,+1]) translate([m*c_to_c/2,e1-D_bushing/2-wall_bushing,25/2+6.5-2.5-10])
            cylinder(h=25,d=D_shaft,center=true);
}

// PhD PCB Holder
//////////////////////////////////////////////////////////////////////////////////
module Holder_PCB_PhD(){
    difference(){
        translate([0,e1/2,23/2-6.5])   cube([23,e1,23], center=true);
        //mounting holes
        for(v=holes) translate(v) rotate([90,0,0])
            cylinder(h=TH,d=M2_screw,center=true);
        //shafts holes
        for(m=[-1,+1]) translate([m*c_to_c/2,e1-D_bushing/2-wall_bushing,0])
            cylinder(h=TH,d=2.2,center=true);
        //bushing
    //    for(m=[-1,+1]) translate([m*c_to_c/2,e1-D_bushing/2-wall_bushing,-TH/2+6.5+e_bushing])
    //        cylinder(h=TH,d=D_bushing+0.1,center=true);
        //trim
        translate([0,TH/2+thickness,-TH/2-6.5+23-10])   cube([TH,TH,TH], center=true);
        //spring hole
        translate([0,d_spring/2+thickness,0])   cylinder(h=TH, d=d_spring, center=true);
    }
    //end-switch arm
    translate([-5.5/2-23/2,(e1+7)/2,-thickness/2-6.5+23])   cube([5.5,e1+7,thickness], center=true);
}


// Hub PCB Holder
//////////////////////////////////////////////////////////////////////////////////
H=h_beam+6.5-2.5;
L=15.5;
module Holder_PCB_Hub() difference(){
    #translate([0,L/2+thickness+delta_holders,-H/2+6.5-2.5])   cube([36,L,H], center=true);
    //shaft holes
    for(m=[-1,+1]) translate([m*c_to_c/2,e1-D_bushing/2-wall_bushing,TH/2+6.5-2.5-10])
        cylinder(h=TH,d=D_shaft,center=true);
    //spring hole
    translate([0,d_spring/2+thickness,0])   cylinder(h=TH, d=d_spring, center=true);
    //motor hole
    translate([0,0,TH/2+4-10]){
        translate([6,10/2+e1-2,0])          cylinder(h=TH, d=10, center=true);
        translate([6-1.5,TH/2+e1-2+1.5,0])    cube([15,TH,TH], center=true);
    }
    //Hub mounting holes
    for(xs=[15,-15], zs=[-3,-13,-23])
        translate([xs,0,6.5-2.5+zs]) rotate([90,0,0])
            cylinder(h=TH, d=M2_screw, center=true);
    //Spectro mounting holes
    for(m=[+1,-1]) translate([m*25/2,8,0])
            cylinder(h=TH, d=M3_screw, center=true);
}


//motor blanket adaptor
//////////////////////////////////////////////////////////////////////////////////
module motor_blanket() difference(){
    translate([0,15-3,0]) cube([13,30,0.6],center=true);
    
    cube([5.75+0.1,1.5+0.1,TH],center=true);
    
    translate([+3.85, 16-8.1, 0]) cylinder(h=TH,d=M2_through,center=true);
    translate([+3.85, 16+0.0, 0]) cylinder(h=TH,d=1.5,center=true);
    translate([+3.85, 16+8.1, 0]) cylinder(h=TH,d=M2_through,center=true);
    translate([-2.85, 16+8.1, 0]) cylinder(h=TH,d=M2_through,center=true);
    
    //trim
    translate([TH/2+4,-TH/2+5.5,0]) cube([TH,TH,TH],center=true); 
}

// Hub PCB
//////////////////////////////////////////////////////////////////////////////////
module PCB_Hub() difference(){
    translate([0,0,2]) cube([35,1.6,47],center=true);
    //mounting holes
    //for(xs=[15,-15], zs=[-5,-20])
    for(xs=[15,-15], zs=[-3,-13,-23])
        translate([xs,0,6.5-2.5+zs]) rotate([90,0,0])
            cylinder(h=TH, d=M2_through, center=true);
    //Motor holes
    translate([6,0,-1]) {
        cube([5.75+0.1,TH,1.5+0.1],center=true);
        translate([+3.85, 0, 16-8.1]) rotate([90,0,0])   cylinder(h=TH,d=M2_screw,center=true);
        translate([+3.85, 0, 16+0.0]) rotate([90,0,0])   cylinder(h=TH,d=1.25,center=true);
        translate([+3.85, 0, 16+8.1]) rotate([90,0,0])   cylinder(h=TH,d=M2_screw,center=true);
        translate([-2.85, 0, 16+8.1]) rotate([90,0,0])   cylinder(h=TH,d=M2_screw,center=true);
    }
    //FFC connector
    translate([-7,0,TH/2-6.5+23])   cube([6,TH,TH],center=true);   
}


////////////////////////////////////////////////////////////////////////////////////
// Assembly
////////////////////////////////////////////////////////////////////////////////////
module detection_unit_assembly(){
    //PCB PhD
    color("green")
    rotate([90,0,0])  translate([-23/2,-6.5,0])
    PCB_PhD();


    //shafts
    color("grey", alpha=0.9)
    Shafts();

    //Holder of PCB PhD
    color("orange", alpha=0.8)
    Holder_PCB_PhD();

    //Holder of PCB Hub
    color("blue",alpha=0.7)
    Holder_PCB_Hub();

    //PCB Hub
    color("green", alpha=0.4)
    //projection() rotate([90,0,180]) //for dxf export
    translate([0,1.6/2+thickness+delta_holders+L,0])
    PCB_Hub();

    //Stepper Motor "blanket"
    color("red",alpha=0.5)
    translate([6,-0.6/2+thickness+delta_holders+L,-1]) rotate([90,0,0])
    motor_blanket();
}

*detection_unit_assembly();


////////////////////////////////////////////////////////////////////////////////////
// 3D printing
////////////////////////////////////////////////////////////////////////////////////
module detection_unit_3Dprint(){
    //Holder of PCB PhD
    color("orange", alpha=0.8) translate([0,0,16.5]) rotate([180,0,0])
    Holder_PCB_PhD();

    //Holder of PCB Hub
    color("blue",alpha=0.7) translate([0,0,h_beam])
    Holder_PCB_Hub();

//    //PCB Hub
//    color("green", alpha=0.4) translate([0,45,1.6/2]) rotate([90,0,0]) //for 3 printing
//    PCB_Hub();
//
//    //Stepper Motor "blanket"
//    color("red",alpha=0.5) translate([28,0,0.6/2]) //for 3 printing
//    motor_blanket();
}

translate([-60,-30,0]) detection_unit_3Dprint();
